"use strict";
/* c8 ignore start */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scriptRunner = exports.runner = exports.isWindows = void 0;
const node_process_1 = __importDefault(require("process"));
const node_path_1 = __importDefault(require("path"));
const get_runtime_js_1 = require("./get-runtime.js");
exports.isWindows = node_process_1.default.platform === 'win32';
const runner = (filename, configs) => {
    const runtime = (0, get_runtime_js_1.getRuntime)(configs);
    // Bun
    if (runtime === 'bun')
        return ['bun'];
    // Deno
    if (runtime === 'deno')
        return [
            'deno',
            'run',
            '--allow-read', // Poku searches for all test files
            '--allow-env', // Poku share the process.env with the `child_process`
            '--allow-run', // Poku CLI
            '--allow-net', // Create Service
        ];
    // Node.js
    return node_path_1.default.extname(filename) === '.ts'
        ? [exports.isWindows ? 'npx.cmd' : 'npx', 'tsx']
        : ['node'];
};
exports.runner = runner;
const scriptRunner = (runner) => {
    // Bun
    if (runner === 'bun')
        return ['bun', 'run'];
    // Deno
    if (runner === 'deno')
        return ['deno', 'task'];
    // Yarn
    if (runner === 'yarn')
        return ['yarn'];
    // PNPM
    if (runner === 'pnpm')
        return ['pnpm', 'run'];
    // Node.js
    return [exports.isWindows ? 'npm.cmd' : 'npm', 'run'];
};
exports.scriptRunner = scriptRunner;
/* c8 ignore stop */
