"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startScript = exports.startService = void 0;
const node_process_1 = __importDefault(require("process"));
const node_child_process_1 = require("child_process");
const runner_js_1 = require("../helpers/runner.js");
const node_path_1 = __importDefault(require("path"));
const list_files_js_1 = require("./list-files.js");
const processes_js_1 = require("./processes.js");
const runningProcesses = new Map();
/* c8 ignore start */
node_process_1.default.once('SIGINT', () => __awaiter(void 0, void 0, void 0, function* () {
    for (const { end, port } of runningProcesses.values()) {
        yield end(port);
    }
}));
/* c8 ignore stop */
const backgroundProcess = (runtime, args, file, options) => new Promise((resolve, reject) => {
    try {
        let isResolved = false;
        const service = (0, node_child_process_1.spawn)(runtime, args, {
            stdio: ['inherit', 'pipe', 'pipe'],
            /* c8 ignore next */
            shell: runner_js_1.isWindows,
            cwd: (options === null || options === void 0 ? void 0 : options.cwd)
                ? (0, list_files_js_1.sanitizePath)(node_path_1.default.normalize(options.cwd))
                : /* c8 ignore next */
                    undefined,
            env: node_process_1.default.env,
            /* c8 ignore next */
            detached: !runner_js_1.isWindows,
            /* c8 ignore next */
            windowsHide: runner_js_1.isWindows,
            timeout: options === null || options === void 0 ? void 0 : options.timeout,
        });
        const PID = service.pid;
        let portBackup;
        /* c8 ignore start */
        const end = (port) => new Promise((resolve) => {
            try {
                runningProcesses.delete(PID);
                if (runner_js_1.isWindows) {
                    processes_js_1.kill.pid(PID);
                    return;
                }
                if (['bun', 'deno'].includes(runtime) ||
                    ['bun', 'deno'].includes(String(options === null || options === void 0 ? void 0 : options.runner))) {
                    node_process_1.default.kill(PID);
                }
                else
                    node_process_1.default.kill(-PID, 'SIGKILL');
                if (port && ['bun', 'deno'].includes(runtime)) {
                    setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                        yield processes_js_1.kill.port(port);
                        resolve(undefined);
                        return;
                    }));
                }
                else {
                    resolve(undefined);
                    return;
                }
            }
            catch (_a) {
                {
                    resolve(undefined);
                    return;
                }
            }
        });
        runningProcesses.set(PID, { end, port: portBackup });
        /* c8 ignore stop */
        /* c8 ignore start */
        service.stdout.on('data', (data) => {
            if (!isResolved && typeof (options === null || options === void 0 ? void 0 : options.startAfter) !== 'number') {
                const stringData = JSON.stringify(String(data));
                if (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'undefined' ||
                    (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'string' &&
                        stringData.includes(options === null || options === void 0 ? void 0 : options.startAfter))) {
                    resolve({ end });
                    clearTimeout(timeout);
                    isResolved = true;
                }
            }
            (options === null || options === void 0 ? void 0 : options.verbose) && console.log(String(data));
        });
        /* c8 ignore stop */
        /* c8 ignore start */
        service.stderr.on('data', (data) => {
            if (!isResolved && typeof (options === null || options === void 0 ? void 0 : options.startAfter) !== 'number') {
                const stringData = JSON.stringify(String(data));
                if (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'undefined' ||
                    (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'string' &&
                        stringData.includes(options === null || options === void 0 ? void 0 : options.startAfter))) {
                    resolve({ end });
                    clearTimeout(timeout);
                    isResolved = true;
                }
            }
            (options === null || options === void 0 ? void 0 : options.verbose) && console.log(String(data));
        });
        /* c8 ignore stop */
        /* c8 ignore stop */
        service.on('error', (err) => {
            end(portBackup);
            reject(`Service failed to start: ${err}`);
        });
        /* c8 ignore stop */
        /* c8 ignore start */
        service.on('close', (code) => {
            if (code !== 0)
                reject(`Service exited with code ${code}`);
        });
        /* c8 ignore stop */
        const timeout = setTimeout(() => {
            if (!isResolved) {
                end(portBackup);
                reject(`createService: Timeout\nFile: ${file}`);
            }
        }, (options === null || options === void 0 ? void 0 : options.timeout) || 60000);
        if (typeof (options === null || options === void 0 ? void 0 : options.startAfter) === 'number') {
            setTimeout(() => {
                if (!isResolved) {
                    resolve({ end });
                    clearTimeout(timeout);
                    isResolved = true;
                }
            }, options.startAfter);
        }
    }
    catch (_a) { }
});
/**
 *
 * Starts a file in a background process
 *
 * Useful for servers, APIs, etc.
 */
const startService = (file, options) => __awaiter(void 0, void 0, void 0, function* () {
    const runtimeOptions = (0, runner_js_1.runner)(file, { platform: options === null || options === void 0 ? void 0 : options.platform });
    const runtime = runtimeOptions.shift();
    const runtimeArgs = [...runtimeOptions, file];
    return yield backgroundProcess(runtime, runtimeArgs, node_path_1.default.normalize((0, list_files_js_1.sanitizePath)(file)), options);
});
exports.startService = startService;
/**
 *
 * Starts a script (package.json) or task (deno.json) in a background process
 *
 * By default, it uses **npm**, but you can costumize it using the `runner` option.
 *
 * Useful for servers, APIs, etc.
 */
const startScript = (script, options) => __awaiter(void 0, void 0, void 0, function* () {
    const runner = (options === null || options === void 0 ? void 0 : options.runner) || 'npm';
    const runtimeOptions = (0, runner_js_1.scriptRunner)(runner);
    const runtime = runtimeOptions.shift();
    const runtimeArgs = [...runtimeOptions, script];
    return yield backgroundProcess(runtime, runtimeArgs, script, Object.assign(Object.assign({}, options), { runner }));
});
exports.startScript = startScript;
