"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.describe = exports.log = void 0;
/* c8 ignore start */
const node_os_1 = require("os");
const format_js_1 = require("../helpers/format.js");
/**
 * By default **Poku** only shows outputs generated from itself.
 * This helper allows you to use an alternative to `console.log` with **Poku**.
 *
 * Need to debug? Just use the [`debug`](https://poku.io/docs/documentation/poku/options/debug) option from `poku`.
 */
const log = (message) => console.log(`\x1b[0m${message}\x1b[0m`);
exports.log = log;
/**
 * On **Poku**, `describe` is just a pretty `console.log` to title your test suites in the terminal.
 */
const describe = (title, options) => {
    const { pad, background, icon } = options || {};
    const message = `${icon || '☰'} ${title}`;
    const noBackground = typeof background === 'boolean' && !background;
    if (noBackground) {
        console.log(`${pad ? node_os_1.EOL : ''}${format_js_1.format.bold(message)}`);
        return;
    }
    console.log(`${pad ? node_os_1.EOL : ''}${format_js_1.format.bg(format_js_1.backgroundColor[typeof background === 'string' ? background : 'grey'], message)}`);
};
exports.describe = describe;
/* c8 ignore stop */
