"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runTestFile = void 0;
const node_process_1 = __importDefault(require("process"));
const node_path_1 = __importDefault(require("path"));
const node_os_1 = require("os");
const node_child_process_1 = require("child_process");
const runner_js_1 = require("../helpers/runner.js");
const indentation_js_1 = require("../configs/indentation.js");
const format_js_1 = require("../helpers/format.js");
const logs_js_1 = require("../helpers/logs.js");
const remove_repeats_js_1 = require("../helpers/remove-repeats.js");
const each_js_1 = require("./each.js");
const files_js_1 = require("../configs/files.js");
const runTestFile = (filePath, configs) => new Promise((resolve) => __awaiter(void 0, void 0, void 0, function* () {
    const runtimeOptions = (0, runner_js_1.runner)(filePath, configs);
    const runtime = runtimeOptions.shift();
    const runtimeArguments = [...runtimeOptions, filePath];
    const fileRelative = node_path_1.default.relative(node_process_1.default.cwd(), filePath);
    const showLogs = !(0, logs_js_1.isQuiet)(configs);
    const showSuccess = (0, logs_js_1.isDebug)(configs);
    const pad = (configs === null || configs === void 0 ? void 0 : configs.parallel) ? '  ' : '    ';
    let output = '';
    const log = () => {
        const outputs = (0, remove_repeats_js_1.removeConsecutiveRepeats)(showSuccess
            ? [output]
            : output.split(/(\r\n|\r|\n)/).filter((current) => {
                if (current.includes('Exited with code'))
                    return false;
                return (/u001b\[0m|(\r\n|\r|\n)/i.test(JSON.stringify(current)) ||
                    current === '');
            }), /(\r\n|\r|\n)|^$/);
        // Remove last EOL
        outputs.length > 1 && outputs.pop();
        if (!showSuccess &&
            /error:/i.test(output) &&
            !/error:/i.test(outputs.join()))
            Object.assign(outputs, [
                ...outputs,
                format_js_1.format.bold(format_js_1.format.fail(`✘ External Error ${format_js_1.format.dim(`› ${fileRelative}`)}`)),
                format_js_1.format.dim('  For detailed diagnostics:'),
                `${format_js_1.format.dim(`    CLI ›`)} rerun with the ${format_js_1.format.bold('--debug')} flag enabled.`,
                `${format_js_1.format.dim(`    API ›`)} set the config option ${format_js_1.format.bold('debug')} to true.`,
                `${format_js_1.format.dim('    RUN ›')} ${format_js_1.format.bold(`${runtime === 'tsx' ? 'npx tsx' : runtime}${runtimeArguments.slice(0, -1).join(' ')} ${fileRelative}`)}`,
            ]);
        const mappedOutputs = outputs.map((current) => `${pad}${current}`);
        if (outputs.length === 1 && outputs[0] === '')
            return;
        console.log(showSuccess ? mappedOutputs.join('') : mappedOutputs.join(node_os_1.EOL));
    };
    const stdOut = (data) => {
        output += String(data);
    };
    if (!(configs === null || configs === void 0 ? void 0 : configs.parallel)) {
        showLogs &&
            console.log(`${indentation_js_1.indentation.test}${format_js_1.format.info(format_js_1.format.dim('●'))} ${format_js_1.format.dim(fileRelative)}`);
    }
    if (!(yield (0, each_js_1.beforeEach)(fileRelative, configs)))
        return false;
    // Export spawn helper is not an option
    const child = (0, node_child_process_1.spawn)(runtime, runtimeArguments, {
        stdio: ['inherit', 'pipe', 'pipe'],
        /* c8 ignore next */
        shell: runner_js_1.isWindows,
        env: Object.assign(Object.assign({}, node_process_1.default.env), { FILE: (configs === null || configs === void 0 ? void 0 : configs.parallel) ? fileRelative : '' }),
    });
    child.stdout.on('data', stdOut);
    child.stderr.on('data', stdOut);
    child.on('close', (code) => __awaiter(void 0, void 0, void 0, function* () {
        if (showLogs)
            log();
        if (!(yield (0, each_js_1.afterEach)(fileRelative, configs)))
            return false;
        const result = code === 0;
        if (result)
            files_js_1.fileResults.success.push(fileRelative);
        else
            files_js_1.fileResults.fail.push(fileRelative);
        resolve(result);
    }));
    child.on('error', (err) => {
        console.log(`Failed to start test: ${filePath}`, err);
        files_js_1.fileResults.fail.push(fileRelative);
        resolve(false);
    });
}));
exports.runTestFile = runTestFile;
